/*
 * Copyright (C) 2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

/*
	<<		op1 << op2		Shift bits of op1 left by distance op2; fills with zero bits on the right-hand side 					<br>
	>>		op1 >> op2		Shift bits of op1 right by distance op2; fills with highest (sign) bit on the left-hand side		<br>
	>>>		op1 >>> op2		Shift bits of op1 right by distance op2; fills with zero bits on the left-hand side					<br>
*/
package estella.lib;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;

/**
* Descriptor Payloads (little-endian)
* @author Daniel Machado
* @since 21/11/2005
*/
public class Payload
{
  private ByteArrayOutputStream bytes;
  private DataOutputStream PONG, QUERY, QUERYHITS;
	private byte[] ping, pong, query, queryhits;
	private Log LOG;
	
	public Payload()
	{
		LOG = new Log("Payload");
	}
	
	public Payload(byte tipo, byte[] payload)
	{
		switch(tipo)
		{
			case (byte)00000000: { this.ping = payload; break; }
			case (byte)00000001: { this.pong = payload; break; }
			case (byte)10000000: { this.query = payload; break; }
			case (byte)10000001: { this.queryhits = payload; break; }
		}
		LOG = new Log("Payload");
	}
			
	/**
	* Payload PING  0x00
	* @return PING - byte zerado
	*/
	public byte[] ping()
	{
		byte[] PING = new byte[1];
		PING[0] = 0x00000000;
		return PING;
	}
		
	/**
	* Payload PONG	0x01
	* Estrutura:
	* 	|porta|host|n_arq_comp|tam_arq_comp| <br>
	*	|--|-----|-----|-----|     <br>  |2|4|4|4|  (bytes)
	* Todos inteiros em little-endian
	* @param porta	 inteiro com a porta 
	* @param host  String com o host
	* @param n_arq_comp  inteiro com o numero de arquivos compartilhados
	* @param tam_arq_comp  inteiro com o tamanho total desses arquivos compartilhados
	* @return byte[]  vetor com os bytes do payload
	*/
	public byte[] pong(int porta, String host, int n_arq_comp, int tam_arq_comp)
	{
		try {
			this.pong = new byte[14];
			
			// Porta (0~1)
			this.pong[0] = (byte) (porta);					// byte 0
			this.pong[1] = (byte) ((porta) >> 8);	// byte 1
			
			// Host (2~5) - endereo de ip - big-endian
			StringTokenizer thost = new StringTokenizer(host, ".");
			this.pong[2] = (byte) (Integer.parseInt(thost.nextToken()));	// byte 2
			this.pong[3] = (byte) (Integer.parseInt(thost.nextToken()));	// byte 3
			this.pong[4] = (byte) (Integer.parseInt(thost.nextToken()));	// byte 4
			this.pong[5] = (byte) (Integer.parseInt(thost.nextToken()));	// byte 5
			
			// Numero de arquivos compartilhados (6-9)
			this.pong[6] = (byte) (n_arq_comp);					// byte 6
			this.pong[7] = (byte) ((n_arq_comp) >> 8);		// byte 7
			this.pong[8] = (byte) ((n_arq_comp) >> 16);	// byte 8
			this.pong[9] = (byte) ((n_arq_comp) >> 24);	// byte 9
			
			// Tamanho total dos arquivos compartilhados (10-13)
			this.pong[10] = (byte) (tam_arq_comp);					// byte 10
			this.pong[11] = (byte) ((tam_arq_comp) >> 8);		// byte 11
			this.pong[12] = (byte) ((tam_arq_comp) >> 16);	// byte 12
			this.pong[13] = (byte) ((tam_arq_comp) >> 24);	// byte 13
			
			// Retorna o payload do tipo pong (vetor de bytes)
			return this.pong;
		}
		catch(Exception FalhaGerarPong)
		{
			return null;
		}
	}
	
	public String pongIp()
	{	
		if(pong.length > 14) {
			LOG.falha("pongIP ~ pong vazio ?");
			return new String((int)(0xFF & pong[2]) + "." + (int)(0xFF & pong[3]) + "." + (int)(0xFF & pong[4]) + "." + (int)(0xFF & pong[5]));
		}
		else
			return null;
	}
	
	public int pongPorta()
	{
		int porta=0;
		porta += (0xFF & (pong[0])) << 0;
		porta += (0xFF & (pong[1])) << 8;

		return porta;
	}
	
	public int byteToint(byte b) {
		return (int) b & 0xFF;
	}
		
	public int pongNumArqComp()
	{
		int numArqComp;
		numArqComp =  (int)(0xFF & pong[6]);
		numArqComp += (int)(0xFF & pong[7]) << 8;
		numArqComp += (int)(0xFF & pong[8]) << 16;
		numArqComp += (int)(0xFF & pong[9]) << 24;
		return numArqComp;
	}
	
	public int pongTamArqComp()
	{
		int pongTamArqComp;
		pongTamArqComp =  (int)(0xFF & pong[10]);
		pongTamArqComp += (int)(0xFF & pong[11]) << 8;
		pongTamArqComp += (int)(0xFF & pong[12]) << 16;
		pongTamArqComp += (int)(0xFF & pong[13]) << 24;
		return pongTamArqComp;
	}
	
	/**
	* Payload Query  0x080
	* @param min_velocidade inteiro com a velocidade minima
	* @param busca String com a sentenca de caracteres a ser buscada
	* @return query - array de bytes com o payload do tipo query
	*/
	public byte[] query(int min_velocidade, String busca)
	{
		try {
			int i=0, j;
			
			// Array QUERY
			byte[] QUERY = new byte[busca.length() + 3];

			// Velocidade minima (0~1)
			QUERY[0] = (byte)min_velocidade;						// byte 1 
			QUERY[1] = (byte)(min_velocidade >> 8);		// byte 2
			
			// String de busca (2~N) aonde N = tamanho da string busca
			for(j=0, i=2; j < busca.getBytes().length; i++, j++)
				QUERY[i] = (byte)busca.charAt(j);

			// Finaliza (N~N+1)
			QUERY[i] = (byte)0x00;
			
			// Retorna o payload do tipo pong (vetor de bytes)
			this.query = QUERY;
			return this.query;
		}
		catch(Exception FalhaGerarQuery)
		{
			return null;
		}
	}
	
	public String queryTermo()
	{
		try {
			byte[] busca = new byte[query.length - 3];
			
			for(int i=2, j=0; j < busca.length; i++, j++)
					busca[j] = (byte)(0xFF & query[i]);
			return new String(busca);
		}
		catch(Exception queryBusca) {
			LOG.falha(queryBusca.getMessage());
			return null;
		}
	}
	
	public int queryMinVelocidade()
	{
		int queryMinVelocidade=0;
		queryMinVelocidade += (0xFF & (query[0])) << 0;
		queryMinVelocidade += (0xFF & (query[1])) << 8;
		return queryMinVelocidade;
	}
	
	/**
	* Payload QueryHits  (0x81)
	* @param 
	*
	*/
	public byte[] queryhits(int hits, int porta, String host, byte[] resultset)
	{
		try {
			int i=0, j=0;
			byte[] queryhits = new byte[11 + resultset.length];
			
			// Numero de hits (0)
			queryhits[0] = (byte)hits;	// byte 0
			
			// Porta (1~2)
			queryhits[1] = (byte)porta;					// byte 1
			queryhits[2] = (byte)(porta >> 8);	// byte 2

			// Host (2~5) - endereo de ip - big-endian
			StringTokenizer thost = new StringTokenizer(host, ".");
			queryhits[3] = (byte) Integer.parseInt(thost.nextToken());	// byte 3
			queryhits[4] = (byte) Integer.parseInt(thost.nextToken());	// byte 4
			queryhits[5] = (byte) Integer.parseInt(thost.nextToken());	// byte 5
			queryhits[6] = (byte) Integer.parseInt(thost.nextToken());	// byte 6
			
			// Velocidade maxima de upload (nao funciona ~ desabilitado)
			queryhits[7] = (byte) 0x00;		// byte 7
			queryhits[8] = (byte) 0x00;		// byte 8
			queryhits[9] = (byte) 0x00;		// byte 9
			queryhits[10] = (byte) 0x00;	// byte 10
			
			// Result Set - conjunto de resultados (11~N)
			for(i=11, j=0; j < resultset.length; j++, i++)
				queryhits[i] = resultset[j];
			
			this.queryhits = queryhits;
			return queryhits;
		}
		catch(Exception FalhaQueryHits)
		{
			LOG.falha(" gerando queryhits " + FalhaQueryHits.getMessage());
			return null;
		}
	}
	
	public int queryhitsNum()
	{
		int numHits=-1;
		try {
			if(queryhits.length > 10)
				numHits = (0xFF & (queryhits[0])) << 0;
			else
				return -1;
		}
		catch(Exception falhaQueryHitsNum) {
			LOG.falha(falhaQueryHitsNum.getMessage());
		}
		return numHits;
	}
	
	public int queryhitsPorta()
	{
		int queryhitsPorta=-1;
		if(queryhits.length > 10) {
			queryhitsPorta=0;
			queryhitsPorta += (0xFF & (queryhits[1])) << 0;
			queryhitsPorta += (0xFF & (queryhits[2])) << 8;
		}
		return queryhitsPorta;
	}
	
	public String queryhitsIp()
	{	
		if(queryhits.length > 10)
			return new String((int)(0xFF & queryhits[3]) + "." + (int)(0xFF & queryhits[4]) + "." + (int)(0xFF & queryhits[5]) + "." + (int)(0xFF & queryhits[6]));
		else
			return null;
	}
	
	public byte[] queryhitsResultset()
	{
		if(this.queryhits.length < 10)
			return null;
		
		byte[] resultset = new byte[this.queryhits.length - 11];
		int i=0, j=0;
		
		for(i=11, j=0; j < resultset.length; i++, j++)
			resultset[j] = queryhits[i];
		return resultset;
	}
	
	public String retornaHit(int numHits, byte[] resultset)
	{
		Constantes CON = new Constantes();
		if(resultset.length < 8 && numHits < 1)
			return null;
		
		String resultados = new String();
		int inicio=0;
		try {
		for(int x=0; x<numHits; x++) {
			int i=0, tamNomeArq=0;
			
			//System.out.print("\n-------------------------------\n1. De: " + inicio + "->");
			int index;
			index =  (int)(0xFF & resultset[inicio+0]);
			index += (int)(0xFF & resultset[inicio+=1]) << 8;
			index += (int)(0xFF & resultset[inicio+=1]) << 16;
			index += (int)(0xFF & resultset[inicio+=1]) << 24;
			//System.out.println(inicio +  " = index: " + index);
			
			//System.out.print("2. De: " + inicio + "->");
			int tamArq;
			tamArq =  (int)(0xFF & resultset[inicio+=1]);
			tamArq += (int)(0xFF & resultset[inicio+=1]) << 8;
			tamArq += (int)(0xFF & resultset[inicio+=1]) << 16;
			tamArq += (int)(0xFF & resultset[inicio+=1]) << 24;		
			//System.out.println(inicio +  " = tamArq: " + tamArq);
			inicio+=1;
			
			for(i=inicio, tamNomeArq=0;  ;i++, tamNomeArq++) {
				if(((int)(0xFF & resultset[i])) ==  0x0000)
					break;
			}
	
			byte[] nomeArq = new byte[tamNomeArq];
			//System.out.print("3. De: " + inicio + "->");
			for(i=0; ;inicio++, i++) {
				if(((int)(0xFF & resultset[inicio])) ==  0x0000)
					break;
				else
				nomeArq[i] = (byte)(0xFF & resultset[inicio]);
			}
			//System.out.println(inicio +  " = nome: " + new String(nomeArq) + " tamtotal: " + (8+nomeArq.length)+"\n----------------------------\n");
			inicio+=1;
			resultados += new String(index+CON.SEPARADOR+tamArq+CON.SEPARADOR+ new String(nomeArq)+CON.SEPARADOR);
		}
		}
		catch(Exception retornaHit) {
			LOG.info(" retornar hit " + retornaHit.getMessage());
		}
		return resultados;
	}
		
	public byte[] hit(int index, int tam_arquivo, String nome)
	{
		byte[] bnome = nome.getBytes();
		byte[] hit = new byte[9+bnome.length];
		int i=0, j=0;
		
		// File index (0~3)
		hit[0] = (byte) index;						// byte 0
		hit[1] = (byte) (index >> 8);			// byte 1
		hit[2] = (byte) (index >> 16);		// byte 2
		hit[3] = (byte) (index >> 24);		// byte 3
		
		// Tamanho do arquivo (4~7)
		hit[4] = (byte) tam_arquivo;						// byte 4
		hit[5] = (byte) (tam_arquivo >> 8);			// byte 5
		hit[6] = (byte) (tam_arquivo >> 16);		// byte 6
		hit[7] = (byte) (tam_arquivo >> 24);		// byte 7
		
		// Nome do arquivo (8~tamanho do nome do arquivo)
		for(j=0, i=8; j < bnome.length; i++, j++)
			hit[i] = bnome[j];
		hit[i] = 0x0000;
		return hit;
	}
	
	public byte[] concatenaHits(byte[] hit1, byte[] hit2)
	{
		String sHit1 = new String(hit1);
		String sHit2 = new String(hit2);
		String resultado = new String(sHit1 + sHit2);
		//System.out.println("hit1: " + sHit1.getBytes().length + " hit2: " + sHit2.getBytes().length + " resutlado: " + resultado.getBytes().length);
		return resultado.getBytes();
	}
}
